package org.drools.mongodb.persistence;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.DisposableBean;

import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.Mongo;


public class MongoStateOps implements StateOps, DisposableBean {
	
	private final String correlatorIdMappingField = "correlatorId";
	private final String correlatorBytesMappingField = "bytes";
	private final String correlatorLastModificationDateMappingField = "lastModificationDate";
	
	private String dbName;
	private String collectionName;
	
	private Mongo mongo;
	private DB    db;
	
	public MongoStateOps(String dbName, Mongo mongo) {
		this.mongo = mongo;
		this.dbName = dbName;
		this.db = this.mongo.getDB(this.dbName);
	}
	
	@Override
	public void persist(String correlatorId, byte[] bytes) {
		DBCollection collection = db.getCollection(collectionName);
		BasicDBObject state = new BasicDBObject();
		state.append(correlatorIdMappingField, correlatorId)
			 .append(correlatorLastModificationDateMappingField, new Date(System.currentTimeMillis()))
			 .append(correlatorBytesMappingField, bytes);
		collection.insert(state);
	}

	@Override
	public void update(String correlatorId, byte[] bytes) {
		BasicDBObject query = new BasicDBObject(correlatorIdMappingField, correlatorId);
		BasicDBObject state = new BasicDBObject();
		state.append(correlatorIdMappingField, correlatorId)
			 .append(correlatorLastModificationDateMappingField, new Date(System.currentTimeMillis()))
			 .append(correlatorBytesMappingField, bytes);
		DBCollection collection = db.getCollection(collectionName);
		collection.update(query, state);
	}
	
	@Override
	public byte[] load(String correlatorId) {
		BasicDBObject query = new BasicDBObject(correlatorIdMappingField, correlatorId);
    	DBCollection collection = db.getCollection(collectionName);
    	
    	BasicDBObject state = (BasicDBObject) collection.findOne(query);
    	
    	
		return (byte[]) state.get(correlatorBytesMappingField);
	}
	
	@Override
	public void destroy() throws Exception {
		if ( this.mongo != null )
			 this.mongo.close();
	}
	
	@Override
	public void clearState(String correlatorId) {
		BasicDBObject query = new BasicDBObject(correlatorIdMappingField, correlatorId);
		DBCollection collection = db.getCollection(collectionName);
		collection.remove(query);
		
	}

	@Override
	public boolean isStateSaved(String correlatorId) {
		BasicDBObject query = new BasicDBObject(correlatorIdMappingField, correlatorId);
		DBCollection collection = db.getCollection(collectionName);
		return ( collection.count( query ) > 0 ) ? true : false;
	}

	@Override
	public List<String> listSavedStates() {
		DBCollection collection = db.getCollection(collectionName);
		DBCursor cursor = collection.find();
		List<String> correlatorStates = new ArrayList<String>();
		while ( cursor.hasNext() ) {
			DBObject state = cursor.next();
			correlatorStates.add(state.get(correlatorIdMappingField).toString());
		}
		return correlatorStates;
	}
	
	public String getCollectionName() {
		return collectionName;
	}

	public void setCollectionName(String collectionName) {
		this.collectionName = collectionName;
	}

	
}
